using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Traysoft.AddTapi;

namespace AutomatedCalls
{
    public partial class Form1 : Form
    {
        delegate void AddListItem(string item);
        AddListItem m_addToLogDelegate;

        public Form1()
        {
            InitializeComponent();
            m_addToLogDelegate = new AddListItem(AddToLog);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                // Set the serial number if you purchased AddTapi.NET
                //TapiApp.SerialNumber = "ABCD-EFGH-IJKL";
                // Initialize AddTapi
                TapiApp.Initialize("AutomatedCalls");
                // Subscribe to events from AddTapi
                TapiApp.TapiError += OnTapiError;
                TapiApp.LineAdded += OnLineAdded;
                TapiApp.LineClosed += OnLineClosed;
                TapiApp.LineRemoved += OnLineRemoved;
                TapiApp.CallDisconnected += OnCallDisconnected;
                // Fill out voice combobox with a list of available text-to-speech voices
                foreach (TapiVoice voice in TapiApp.Voices)
                {
                    comboBoxVoice.Items.Add(voice);
                }
                // Show line names in the combobox
                UpdateLinesCombobox();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                TapiApp.Shutdown();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonDial_Click(object sender, EventArgs e)
        {
            // Check if a line was selected
            TapiLine line = (TapiLine)comboBoxLine.SelectedItem;
            if (line == null)
            {
                MessageBox.Show("Please select a line first.");
                return;
            }
            // Check if a number was entered
            string number = textBoxNumber.Text;
            if (number == null || number.Length == 0)
            {
                MessageBox.Show("Please enter a number to call.");
                return;
            }
            try
            {
                // Check if this line supports voice functions (playing wav files and speech)
                if (!line.SupportsVoice)
                {
                    MessageBox.Show("This line does not support audio playback/recording and cannot be used to play wav files.\n" +
                                "If this is a voice modem, make sure you have installed TAPI-compatible voice driver for the modem.\n\n" +
                                "Outgoing call will be placed in interactive mode and terminated 30 seconds after the call is connected.");
                }
                // Open selected line for outgoing calls if it wasn't already open
                if (!line.IsOpen) line.Open(false, CallHandler);
                line.SpeechVoice = (TapiVoice)comboBoxVoice.SelectedItem;
                // Set no answer timeout to 30 seconds (approx. 5 rings)
                line.NoAnswerTimeout = 30;
                // Dial entered number without the dialing rules
                line.Dial(number, false);
                string msg = String.Format("Dialing {0} on line '{1}'", number, line.Name);
                AddToLog(msg);
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // This method will be called when outgoing call is connected.
        // All call flow processing is performed in this method.
        // Please note that voice modems don't support answer detection and
        // this method will be called before the call is answered when the phone is still ringing.
        // This method is called on a separate thread and is executed in the background.
        // Several copies of this method can be executed at the same time (one for each line).
        // If you call methods of your class, make sure those methods are thread-safe.
        // WinForms controls can't be accessed directly in this method.
        private void CallHandler(TapiCall call)
        {
            try
            {
                string msg = String.Format("Connected call to {0}.", call.CalledID);
                // Because it's another thread, we have to use Invoke to access UI controls
                listBoxLog.Invoke(m_addToLogDelegate, msg);

                // If the line doesn't support voice, wait 30 seconds then terminate the call
                if (!call.Line.SupportsVoice)
                {
                    System.Threading.Thread.Sleep(30000);
                    return;
                }

                // Try to deliver a message, up to 5 times.
                for (int retry = 0; retry < 5; retry++)
                {
                    // Play a message:
                    // "Hello! This is a message from Traysoft telephony demo."
                    // "Please press 1 to listen to the message."
                    call.Play("..\\..\\Wavs\\hello.wav");
                    call.Play("..\\..\\Wavs\\press1.wav");
                    // Wait for a digit (up to 10 seconds)
                    string digit = call.WaitForDigit(10);
                    if (digit != null && digit == "1")
                    {
                        // Button 1 was pressed, speak using selected voice
                        if (call.Line.SpeechVoice != null)
                        {
                            msg = String.Format("You have selected {0} voice.",
                                call.Line.SpeechVoice.Name);
                        }
                        else
                        {
                            msg = "You have selected default computer voice.";
                        }
                        call.Speak(msg, true);
                        // Wait until speaking is finished
                        call.WaitUntilDone(10);
                        msg = String.Format("Message delivered to {0}.", call.CalledID);
                        // Because it's another thread, we have to use Invoke to access UI controls
                        listBoxLog.Invoke(m_addToLogDelegate, msg);
                        break;
                    }
                }

                // Play goodbye message:
                // "Thank you for using Traysoft telephony demo. Goodbye, and have a good day."
                call.Play("..\\..\\Wavs\\goodbye.wav", true);
                // Wait until playing is finished
                call.WaitUntilDone(10);
            }
            catch (TapiDisconnectException)
            {
                // This exception is thrown if the other party hung up before we called Disconnect().
                listBoxLog.Invoke(m_addToLogDelegate, "The called party hung up.");
            }
            catch (Exception exc)
            {
                string msg = String.Format("Exception in CallHandler! {0}", exc.Message);
                listBoxLog.Invoke(m_addToLogDelegate, msg);
            }
            finally
            {
                // Disconnect the call to release the line
                call.Disconnect();
            }
        }

        // This event handler is called when the call has been disconnected.
        // After the current call is disconnected, another call can be placed on the line.
        void OnCallDisconnected(object sender, TapiEventArgs args)
        {
            string msg = String.Format("Disconnected call to {0} on line '{1}'",
                args.Call.CalledID, args.Line.Name);
            // Check why the call was disconnected
            switch(args.Call.DisconnectMode)
            {
                case TapiDisconnectMode.Busy:
                    msg += ", the number was busy";
                    break;
                case TapiDisconnectMode.NoAnswer:
                    msg += ", there was no answer";
                    break;
                case TapiDisconnectMode.NoDialTone:
                    msg += ", there was no dialtone";
                    break;
                case TapiDisconnectMode.Error:
                    msg += ", an error occurred";
                    break;
            }
            AddToLog(msg);
        }

        // This event handler is called when AddTapi error occurs asynchronously
        void OnTapiError(object sender, TapiErrorEventArgs args)
        {
            // Display error in the log
            string msg;
            if (args.Line != null)
            {
                msg = String.Format("TapiError event, line '{0}'. {1}",
                        args.Line.Name, args.Message);
            }
            else
            {
                msg = String.Format("TapiError event. {0}", args.Message);
            }
            AddToLog(msg);
        }

        // This event handler is called when new line has been added
        // while application is running, for example USB modem was connected.
        void OnLineAdded(object sender, TapiEventArgs args)
        {
            UpdateLinesCombobox();
        }

        // This event handler is called when the line was forcibly closed
        // by Windows because of hardware error or configuration change.
        void OnLineClosed(object sender, TapiEventArgs args)
        {
            // Display message in the log
            string msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.",
                            args.Line.Name);
            AddToLog(msg);
            UpdateLinesCombobox();
        }

        // This event handler is called when new line has been removed
        // while application is running, for example USB modem was disconnected.
        void OnLineRemoved(object sender, TapiEventArgs args)
        {
            UpdateLinesCombobox();
        }

        private void AddToLog(string msg)
        {
            int ind = listBoxLog.Items.Add(msg);
            // Ensure that the added item is visible
            listBoxLog.TopIndex = ind;
        }

        private void UpdateLinesCombobox()
        {
            comboBoxLine.Items.Clear();
            foreach(TapiLine line in TapiApp.Lines)
            {
                comboBoxLine.Items.Add(line);
            }
        }
    }
}
