using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Threading;
using Traysoft.AddTapi;

namespace CallerID
{
    public partial class Form1 : Form
    {
        delegate void AddListItem(string item);
        AddListItem m_addToLogDelegate;

        public Form1()
        {
            InitializeComponent();
            m_addToLogDelegate = new AddListItem(AddToLog);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                // Set the serial number if you purchased AddTapi.NET
                //TapiApp.SerialNumber = "ABCD-EFGH-IJKL";
                // Initialize AddTapi
                TapiApp.Initialize("CallerID");
                // Subscribe to events from AddTapi
                TapiApp.TapiError += OnTapiError;
                TapiApp.LineAdded += OnLineAdded;
                TapiApp.LineClosed += OnLineClosed;
                TapiApp.LineRemoved += OnLineRemoved;
                TapiApp.IncomingCall += OnIncomingCall;
                TapiApp.OutgoingCall += OnOutgoingCall;
                TapiApp.CallConnected += OnCallConnected;
                TapiApp.CallDisconnected += OnCallDisconnected;
                // Show line names
                UpdateLinesCombobox();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                TapiApp.Shutdown();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonOpen_Click(object sender, EventArgs e)
        {
            // Check if a line was selected
            TapiLine line = (TapiLine)comboBoxLine.SelectedItem;
            if (line == null)
            {
                MessageBox.Show("Please select a line first.");
                return;
            }
            // Check if the line is already open
            if (line.IsOpen)
            {
                AddToLog("Already monitoring line " + line.Name);
                return;
            }
            // Open selected line
            try
            {
                line.RingsToAnswer = 0;
                line.Open(true, null);
                AddToLog("Started monitoring line " + line.Name);
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            // Check if a line was selected
            TapiLine line = (TapiLine)comboBoxLine.SelectedItem;
            if (line == null)
            {
                MessageBox.Show("Please select a line first.");
                return;
            }
            // Close selected line
            try
            {
                if (line.IsOpen)
                {
                    line.Close();
                }
                AddToLog("Stopped monitoring line " + line.Name);
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // This event handler is called when a new incoming call is received.
        void OnIncomingCall(object sender, TapiEventArgs args)
        {
            // Display message in the log
            string msg;
            msg = String.Format("Incoming call from {0} {1} on line '{2}'.",
                       args.Call.CallerIDName, args.Call.CallerID, args.Line.Name);
            AddToLog(msg);
        }

        // This event handler is called when a new outgoing call is detected.
        void OnOutgoingCall(object sender, TapiEventArgs args)
        {
            // Display message in the log
            string msg;
            msg = String.Format("Outgoing call to {0} on line '{1}'.",
                       args.Call.CalledID, args.Line.Name);
            AddToLog(msg);
        }

        // This event handler is called when the call has been connected.
        void OnCallConnected(object sender, TapiEventArgs args)
        {
            string msg;
            if (args.Call.Direction == TapiCallDirection.Incoming)
            {
                msg = String.Format("Connected incoming call from {0} on line '{1}'",
                           args.Call.CallerID, args.Line.Name);
            }
            else
            {
                msg = String.Format("Connected outgoing call to {0} on line '{1}'",
                           args.Call.CalledID, args.Line.Name);
            }        
            AddToLog(msg);
        }

        // This event handler is called when the call has been disconnected.
        void OnCallDisconnected(object sender, TapiEventArgs args)
        {
            string msg;
            msg = String.Format("Disconnected the call on line '{0}'", args.Line.Name);
            AddToLog(msg);
        }

        // This event handler is called when AddTapi error occurs asynchronously
        void OnTapiError(object sender, TapiErrorEventArgs args)
        {
            // Display error in the log
            string msg;
            if (args.Line != null)
            {
                msg = String.Format("TapiError event, line '{0}'. {1}",
                        args.Line.Name, args.Message);
            }
            else
            {
                msg = String.Format("TapiError event. {0}", args.Message);
            }
            AddToLog(msg);
        }

        // This event handler is called when new line has been added
        // while application is running, for example USB modem was connected.
        void OnLineAdded(object sender, TapiEventArgs args)
        {
            UpdateLinesCombobox();
        }

        // This event handler is called when the line was forcibly closed
        // by Windows because of hardware error or configuration change.
        void OnLineClosed(object sender, TapiEventArgs args)
        {
            // Display message in the log
            string msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.",
                            args.Line.Name);
            AddToLog(msg);
            UpdateLinesCombobox();
        }

        // This event handler is called when new line has been removed
        // while application is running, for example USB modem was disconnected.
        void OnLineRemoved(object sender, TapiEventArgs args)
        {
            UpdateLinesCombobox();
        }

        private void AddToLog(string msg)
        {
            int ind = listBoxLog.Items.Add(msg);
            // Ensure that the added item is visible
            listBoxLog.TopIndex = ind;
        }

        void UpdateLinesCombobox()
        {
            comboBoxLine.Items.Clear();
            foreach (TapiLine line in TapiApp.Lines)
            {
                comboBoxLine.Items.Add(line);
            }
        }
    }
}