using System;
using System.IO;
using System.Threading;
using Microsoft.Win32.SafeHandles;
using Traysoft.AddTapi;

namespace DataCall
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                // Set the serial number if you purchased AddTapi.NET
                //TapiApp.SerialNumber = "ABCD-EFGH-IJKL";
                // Initialize AddTapi
                TapiApp.Initialize("DataCall");
                // Subscribe to events from AddTapi
                TapiApp.TapiError += OnTapiError;
                // List all available lines
                string[] selection = new string[TapiApp.Lines.Count];
                for (int i = 0; i < selection.Length; i++)
                {
                    selection[i] = TapiApp.Lines[i].Name;
                }
                int selected = Select("Please select line:", selection);
                TapiLine selectedLine = TapiApp.Lines[selected];
                // Ask for a phone number to dial
                Console.Write("Please enter a number to dial: ");
                string number = Console.ReadLine().Trim();
                // Open selected line for outgoing calls
                selectedLine.Open(false, DataCallHandler, TapiMediaMode.DataModem);
                // Make outgoing data call
                Console.WriteLine("Dialing {0}", number);
                TapiCall call = selectedLine.Dial(number, false, TapiMediaMode.DataModem);
                // Wait until the call is disconnected
                do
                {
                    Thread.Sleep(1000);
                } while (call.State != TapiCallState.Disconnected &&
                            call.State != TapiCallState.Idle);
                selectedLine.Close();
                Console.WriteLine("\nPress any key to exit.");
                Console.ReadKey(true);
            }
            catch (TapiException exc)
            {
                Console.WriteLine("\n\nTapiException!\n{0}\n", exc.Message);
                Console.ReadKey(false);
            }
            catch (Exception exc)
            {
                Console.WriteLine("\n\nException!\n{0}\n", exc.Message);
                Console.ReadKey(false);
            }
            finally
            {
                TapiApp.Shutdown();
            }
        }

        // This method will be called when outgoing call is connected.
        // All call processing is performed in this method.
        // This method is called on a separate thread and is executed in the background.
        // Several copies of this method can be executed at the same time (one for each line).
        // If you call methods of your class, make sure those methods are thread-safe.
        static void DataCallHandler(TapiCall tapiCall)
        {
            SafeFileHandle commHandle = null;
            try
            {
                Console.WriteLine("Data connection has been established.");
                Console.WriteLine("Starting terminal session. Press <Esc> to disconnected the call.\n");                
                // Obtain COM port handle
                commHandle = new SafeFileHandle(tapiCall.GetCommHandle(), true);
                // Create FileStream object associated with the handle
                using (FileStream fs = new FileStream(commHandle, FileAccess.ReadWrite, 4096, true))
                {
                    TerminalSession(fs, tapiCall);
                }
                Console.WriteLine("\n\nTerminal session ended.");
            }
            catch (Exception exc)
            {
                Console.WriteLine("\n\nException in DataCallHandler: {0}\n", exc.Message);
            }
            finally
            {
                // Release the COM handle
                if (commHandle != null) commHandle.Close();
                // Disconnect the call to release the line
                tapiCall.Disconnect();
            }
        }

        // Runs interactive terminal session until <Esc> is pressed
        static void TerminalSession(FileStream fs, TapiCall tapiCall)
        {
            // Start asynchronous reading from the stream
            BeginAsyncRead(fs);
            // While the call is connected,
            // send all pressed keys to the stream
            while (tapiCall.State == TapiCallState.Connected)
            {
                if (Console.KeyAvailable)
                {
                    ConsoleKeyInfo cki = Console.ReadKey();
                    // <Esc> pressed, terminate the session
                    if (cki.Key == ConsoleKey.Escape) break;
                    if (cki.KeyChar > 0 && cki.KeyChar < 256)
                    {
                        // Write the character to the stream
                        fs.WriteByte((byte)cki.KeyChar);
                        // Flush FileStream buffer to make sure
                        // entered character is transmitted
                        // Normally you would call Flush() after sending
                        // a block of data
                        fs.Flush();
                    }
                }
                Thread.Sleep(10);
            }
        }

        // Holds read data and FileStream object for async read
        class AsyncReadState
        {
            public byte[] Data;
            public FileStream Stream;
        }

        // Allocates memory for read data and begins async read operation
        static void BeginAsyncRead(FileStream fs)
        {
            try
            {
                AsyncReadState state = new AsyncReadState();
                state.Data = new byte[1];
                state.Stream = fs;
                fs.BeginRead(state.Data, 0, 1, AsyncReadCallback, state);
            }
            catch (OperationCanceledException)
            {
                // Call has been disconnected
                // We don't need to do anything
            }
        }

        // This method is called when async read operation completes
        static void AsyncReadCallback(IAsyncResult ar)
        {
            try
            {
                // Obtain read data
                AsyncReadState state = (AsyncReadState)ar.AsyncState;
                int count = state.Stream.EndRead(ar);
                // Write read data to the console
                for (int i = 0; i < count; i++) Console.Write((char)state.Data[i]);
                // Begin another async read operation
                BeginAsyncRead(state.Stream);
            }
            catch (Exception exc)
            {
                Console.WriteLine("\n\nException in AsyncReadCallback: {0}", exc.Message);
            }
        }

        // This event handler is called when AddTapi error occurs asynchronously
        static void OnTapiError(object sender, TapiErrorEventArgs e)
        {
            Console.WriteLine("\n\nTapi error: {0}\n", e.Message);
        }

        // Utility function: allows user to select an item from a list
        static int Select(string prompt, string[] selection)
        {
            Console.WriteLine(prompt);
            for (int i = 0; i < selection.Length; i++)
            {
                Console.WriteLine("{0} - {1}", i + 1, selection[i]);
            }

            int selected = 0;
            do
            {
                char c = Console.ReadKey(true).KeyChar;
                if (Char.IsDigit(c))
                {
                    string s = "" + c;
                    selected = Int32.Parse(s);
                    if (selected < 1 || selected > selection.Length) selected = 0;
                }
            } while (selected == 0);
            return selected - 1;
        }
    }
}
