﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Traysoft.AddTapi;

namespace Dial
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Set the serial number if you purchased AddTapi.NET
            //TapiApp.SerialNumber = "ABCD-EFGH-IJKL";
            // Initialize AddTapi
            TapiApp.Initialize("Dial");
            // Subscribe to events from AddTapi
            TapiApp.TapiError += OnTapiError;
            TapiApp.LineAdded += OnLineAdded;
            TapiApp.LineClosed += OnLineClosed;
            TapiApp.LineRemoved += OnLineRemoved;
            TapiApp.CallConnected += OnCallConnected;
            TapiApp.CallDisconnected += OnCallDisconnected;
            // Show line names in the combobox
            UpdateLinesCombobox();
        }


        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                TapiApp.Shutdown();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonDial_Click(object sender, EventArgs e)
        {
            // Check if a line was selected
            TapiLine line = (TapiLine)comboBoxLine.SelectedItem;
            if (line == null)
            {
                MessageBox.Show("Please select a line first.");
                return;
            }
            // Check if a number was entered
            string number = textBoxNumber.Text;
            if (number == null || number.Length == 0)
            {
                MessageBox.Show("Please enter a number to call.");
                return;
            }
            try
            {
                // Open selected line for outgoing calls if it wasn't already open
                if (!line.IsOpen) line.Open(false, null, TapiMediaMode.InteractiveVoice);
                // Dial entered number without the dialing rules
                line.Dial(number, false);
                string msg = String.Format("Dialing {0} on line '{1}'", number, line.Name);
                AddToLog(msg);
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            // Check if a line was selected
            TapiLine line = (TapiLine)comboBoxLine.SelectedItem;
            if (line == null) return;
            try
            {
                // Find a call that has not been disconnected and disconnect it
                foreach (TapiCall call in line.Calls)
                {
                    if (call.State != TapiCallState.Idle && call.State != TapiCallState.Disconnected)
                    {
                        string msg = String.Format("Disconnecting call to {0} on line '{1}'", call.CalledID, line.Name);
                        AddToLog(msg);
                        call.Disconnect();
                    }
                }
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // This event handler is called when the call has been connected to the other party.
        void OnCallConnected(object sender, TapiEventArgs args)
        {
            string msg = String.Format("Connected call to {0} on line '{1}', pick up the handset to talk.",
                args.Call.CalledID, args.Line.Name);
        }

        // This event handler is called when the call has been disconnected.
        // After the current call is disconnected, another call can be placed on the line.
        void OnCallDisconnected(object sender, TapiEventArgs args)
        {
            string msg = String.Format("Disconnected call to {0} on line '{1}'",
                args.Call.CalledID, args.Line.Name);
            // Check why the call was disconnected
            switch (args.Call.DisconnectMode)
            {
                case TapiDisconnectMode.Busy:
                    msg += ", the number was busy";
                    break;
                case TapiDisconnectMode.NoAnswer:
                    msg += ", there was no answer";
                    break;
                case TapiDisconnectMode.NoDialTone:
                    msg += ", there was no dialtone";
                    break;
                case TapiDisconnectMode.Error:
                    msg += ", an error occurred";
                    break;
            }
            AddToLog(msg);

            // Use TapiCall.Disconnect to release the line
            try
            {
                args.Call.Disconnect();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // This event handler is called when AddTapi error occurs asynchronously
        void OnTapiError(object sender, TapiErrorEventArgs args)
        {
            // Display error in the log
            string msg;
            if (args.Line != null)
            {
                msg = String.Format("TapiError event, line '{0}'. {1}",
                        args.Line.Name, args.Message);
            }
            else
            {
                msg = String.Format("TapiError event. {0}", args.Message);
            }
            AddToLog(msg);
        }

        // This event handler is called when new line has been added
        // while application is running, for example USB modem was connected.
        void OnLineAdded(object sender, TapiEventArgs args)
        {
            UpdateLinesCombobox();
        }

        // This event handler is called when the line was forcibly closed
        // by Windows because of hardware error or configuration change.
        void OnLineClosed(object sender, TapiEventArgs args)
        {
            // Display message in the log
            string msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.",
                            args.Line.Name);
            AddToLog(msg);
            UpdateLinesCombobox();
        }

        // This event handler is called when new line has been removed
        // while application is running, for example USB modem was disconnected.
        void OnLineRemoved(object sender, TapiEventArgs args)
        {
            UpdateLinesCombobox();
        }

        private void AddToLog(string msg)
        {
            int ind = listBoxLog.Items.Add(msg);
            // Ensure that the added item is visible
            listBoxLog.TopIndex = ind;
        }

        private void UpdateLinesCombobox()
        {
            comboBoxLine.Items.Clear();
            foreach (TapiLine line in TapiApp.Lines)
            {
                comboBoxLine.Items.Add(line);
            }
        }
    }
}
