using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Traysoft.AddTapi;

namespace VoiceMail
{
    public partial class Form1 : Form
    {
        delegate void AddListItem(string item);
        AddListItem m_addToLogDelegate;

        public Form1()
        {
            InitializeComponent();
            m_addToLogDelegate = new AddListItem(AddToLog);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                // Set the serial number if you purchased AddTapi.NET
                //TapiApp.SerialNumber = "ABCD-EFGH-IJKL";
                // Initialize AddTapi
                TapiApp.Initialize("VoiceMail");
                // Subscribe to events from AddTapi
                TapiApp.TapiError += OnTapiError;
                TapiApp.LineAdded += OnLineAdded;
                TapiApp.LineClosed += OnLineClosed;
                TapiApp.LineRemoved += OnLineRemoved;
                TapiApp.IncomingCall += OnIncomingCall;
                // Update controls to show line names
                UpdateFormControls();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                TapiApp.Shutdown();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // This method will be called when incoming call is answered and connected.
        // All call flow processing is performed in this method.
        // This method is called on a separate thread and is executed in the background.
        // Several copies of this method can be executed at the same time (one for each line).
        // If you call methods of your class, make sure those methods are thread-safe.
        // WinForms controls can't be accessed directly in this method.
        private void CallHandler(TapiCall call)
        {
            try
            {
                string msg = String.Format("Answered call from {0}.", call.CallerID);
                // Because it's another thread, we have to use Invoke to access UI controls
                listBoxLog.Invoke(m_addToLogDelegate, msg);

                // Play a welcome message:
                // "Welcome to Traysoft telephony demo!"
                call.Play("..\\..\\Wavs\\welcome.wav");
                // Ask caller to leave a message:
                // "Please leave a message after the beep. Press the pound key when you are finished."
                call.Play("..\\..\\Wavs\\leavemsg.wav");
                call.Play("..\\..\\Wavs\\beep.wav");
                // Wait until playback is completed (30 seconds maximum).
                call.WaitUntilDone(30);

                // Generate filename for the voice mail file based on current date/time
                DateTime now = DateTime.Now;
                string filename = String.Format("Voicemail_{0}_{1}_{2}_{3}_{4}_{5}.wav",
                        now.Year, now.Month, now.Day, now.Hour, now.Minute, now.Second);

                // Most telephony devices do not support playback and recording at the same time
                // Call StopPlayback to make sure playback is stopped.
                call.StopPlayback();
                call.StartRecording(filename);

                msg = "Recording message " + filename;
                // Because it's another thread, we have to use Invoke to access UI controls
                listBoxLog.Invoke(m_addToLogDelegate, msg);

                // Wait for a button to stop recording (60 seconds maximum)
                call.WaitForDigit(1200);
                call.StopRecording();

                // Ask caller if she wants to listen to the recorded message:
                // "Press 1 to listen to your message. Press * to disconnect".
                call.Play("..\\..\\Wavs\\listenmsg.wav");
                if (call.WaitForDigit(10) == "1")
                {
                    // Play recorded message back to the caller
                    call.Play(filename, true);
                    call.WaitUntilDone(60);
                }

                // Play goodbye message:
                // "Thank you for using Traysoft telephony demo. Goodbye, and have a good day."
                call.Play("..\\..\\Wavs\\goodbye.wav", true);
                // Wait until playing is finished
                call.WaitUntilDone(10);
            }
            catch (TapiDisconnectException)
            {
                // This exception is thrown if the caller hung up before we called Disconnect().
                listBoxLog.Invoke(m_addToLogDelegate, "The caller hung up.");
            }
            catch (Exception exc)
            {
                string msg = String.Format("Exception in CallHandler! {0}", exc.Message);
                listBoxLog.Invoke(m_addToLogDelegate, msg);
            }
            finally
            {
                // Disconnect the call to clear up the line
                call.Disconnect();
            }
        }

        private void OpenLine(int lineIndex)
        {
            try
            {
                // If specified line doesn't exist, don't do anything
                if (lineIndex >= TapiApp.Lines.Count) return;
                // Open specified line for incoming calls
                TapiLine line = TapiApp.Lines[lineIndex];
                // Check if this line supports voice functions (playing wav files and speech)
                if (!line.SupportsVoice)
                {
                    MessageBox.Show("This line does not support audio playback/recording and cannot be used to play messages to the callers.\n" +
                                "If this is a voice modem, make sure you have installed TAPI-compatible voice driver for the modem.\n");
                    return;
                }
                line.Open(true, CallHandler);
                line.RingsToAnswer = (int)numericUpDownRings.Value;
                UpdateFormControls();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CloseLine(int lineIndex)
        {
            try
            {
                // If specified line doesn't exist, don't do anything
                if (lineIndex >= TapiApp.Lines.Count) return;
                // Close specified line
                TapiLine line = TapiApp.Lines[lineIndex];
                line.Close();
                UpdateFormControls();
            }
            catch (TapiException exc)
            {
                MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // This event handler is called when AddTapi error occurs asynchronously
        void OnTapiError(object sender, TapiErrorEventArgs args)
        {
            // Display error in the log
            string msg;
            if (args.Line != null)
            {
                msg = String.Format("TapiError event, line '{0}'. {1}",
                        args.Line.Name, args.Message);
            }
            else
            {
                msg = String.Format("TapiError event. {0}", args.Message);
            }
            AddToLog(msg);
        }

        // This event handler is called when new line has been added
        // while application is running, for example USB modem was connected.
        void OnLineAdded(object sender, TapiEventArgs args)
        {
            UpdateFormControls();
        }

        // This event handler is called when the line was forcibly closed
        // by Windows because of hardware error or configuration change.
        void OnLineClosed(object sender, TapiEventArgs args)
        {
            // Display message in the log
            string msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.",
                            args.Line.Name);
            AddToLog(msg);
            UpdateFormControls();
        }

        // This event handler is called when new line has been removed
        // while application is running, for example USB modem was disconnected.
        void OnLineRemoved(object sender, TapiEventArgs args)
        {
            UpdateFormControls();
        }

        // This event handler is called when new call is received.
        // You can use this handler for logging, UI updates etc.
        void OnIncomingCall(object sender, TapiEventArgs args)
        {
            // Display message in the log
            string msg = String.Format("Incoming call from {0} on line '{1}'.",
                            args.Call.CallerID, args.Line.Name);
            AddToLog(msg);
        }

        // This method displays line names and enables/disables Open and Close buttons
        private void UpdateFormControls()
        {
            textBoxLine1.Text = "";
            buttonLine1Open.Enabled = false;
            buttonLine1Close.Enabled = false;
            if (TapiApp.Lines.Count >= 1)
            {
                // Line 1 is present
                TapiLine line = TapiApp.Lines[0];
                textBoxLine1.Text = line.Name;
                if (line.IsOpen) buttonLine1Close.Enabled = true;
                else buttonLine1Open.Enabled = true;
            }

            textBoxLine2.Text = "";
            buttonLine2Open.Enabled = false;
            buttonLine2Close.Enabled = false;
            if (TapiApp.Lines.Count >= 2)
            {
                // Line 2 is present
                TapiLine line = TapiApp.Lines[1];
                textBoxLine2.Text = line.Name;
                if (line.IsOpen) buttonLine2Close.Enabled = true;
                else buttonLine2Open.Enabled = true;
            }

            textBoxLine3.Text = "";
            buttonLine3Open.Enabled = false;
            buttonLine3Close.Enabled = false;
            if (TapiApp.Lines.Count >= 3)
            {
                // Line 3 is present
                TapiLine line = TapiApp.Lines[2];
                textBoxLine3.Text = line.Name;
                if (line.IsOpen) buttonLine3Close.Enabled = true;
                else buttonLine3Open.Enabled = true;
            }

            textBoxLine4.Text = "";
            buttonLine4Open.Enabled = false;
            buttonLine4Close.Enabled = false;
            if (TapiApp.Lines.Count >= 4)
            {
                // Line 4 is present
                TapiLine line = TapiApp.Lines[3];
                textBoxLine4.Text = line.Name;
                if (line.IsOpen) buttonLine4Close.Enabled = true;
                else buttonLine4Open.Enabled = true;
            }

            textBoxLine5.Text = "";
            buttonLine5Open.Enabled = false;
            buttonLine5Close.Enabled = false;
            if (TapiApp.Lines.Count >= 5)
            {
                // Line 5 is present
                TapiLine line = TapiApp.Lines[4];
                textBoxLine5.Text = line.Name;
                if (line.IsOpen) buttonLine5Close.Enabled = true;
                else buttonLine5Open.Enabled = true;
            }
        }

        private void numericUpDownRings_ValueChanged(object sender, EventArgs e)
        {
            // Number of rings to answer has changed, assign new value to all lines
            foreach (TapiLine line in TapiApp.Lines)
            {
                line.RingsToAnswer = (int)numericUpDownRings.Value;
            }
        }

        private void AddToLog(string msg)
        {
            int ind = listBoxLog.Items.Add(msg);
            // Ensure that the added item is visible
            listBoxLog.TopIndex = ind;
        }

        private void buttonLine1Open_Click(object sender, EventArgs e)
        {
            OpenLine(0);
        }

        private void buttonLine2Open_Click(object sender, EventArgs e)
        {
            OpenLine(1);
        }

        private void buttonLine3Open_Click(object sender, EventArgs e)
        {
            OpenLine(2);
        }

        private void buttonLine4Open_Click(object sender, EventArgs e)
        {
            OpenLine(3);
        }

        private void buttonLine5Open_Click(object sender, EventArgs e)
        {
            OpenLine(4);
        }

        private void buttonLine1Close_Click(object sender, EventArgs e)
        {
            CloseLine(0);
        }

        private void buttonLine2Close_Click(object sender, EventArgs e)
        {
            CloseLine(1);
        }

        private void buttonLine3Close_Click(object sender, EventArgs e)
        {
            CloseLine(2);
        }

        private void buttonLine4Close_Click(object sender, EventArgs e)
        {
            CloseLine(3);
        }

        private void buttonLine5Close_Click(object sender, EventArgs e)
        {
            CloseLine(4);
        }
    }
}
